@@ -9,6 +9,7 @@ from repoze.zodbconn.datatypes import TRUETYPES
 
 from ZODB.FileStorage.FileStorage import FileStorage
 from ZODB.DemoStorage import DemoStorage
+from ZODB.MappingStorage import MappingStorage
 from ZODB.blob import BlobStorage
 from ZODB.DB import DB
 import ZConfig
@@ -61,6 +62,30 @@ class Resolver(object):
         new.update(newkw)
         return new
 
+class MappingStorageURIResolver(Resolver):
+    _int_args = ('connection_cache_size', 'connection_pool_size')
+    _string_args = ('database_name',)
+    _bytesize_args = ()
+    def __call__(self, uri):
+        prefix, rest = uri.split('memory://', 1)
+        result = rest.split('?', 1)
+        if len(result) == 1:
+            name = result[0]
+            query = ''
+        else:
+            name, query = result
+        kw = dict(cgi.parse_qsl(query))
+        kw = self.interpret_kwargs(kw)
+        dbkw = get_dbkw(kw)
+        args = (name,)
+        dbitems = dbkw.items()
+        dbitems.sort()
+        key = (args, tuple(dbitems))
+        def factory():
+            storage = MappingStorage(*args)
+            return DB(storage, **dbkw)
+        return key, args, kw, factory
+        
 class FileStorageURIResolver(Resolver):
     _int_args = ('create', 'read_only', 'demostorage', 'connection_cache_size',
                  'connection_pool_size')
@@ -134,9 +159,9 @@ class ClientStorageURIResolver(Resolver):
     _bytesize_args = ('cache_size', )
 
     def __call__(self, uri):
+        # urlparse doesnt understand zeo URLs so force to something that doesn't break
+        uri = uri.replace('zeo://', 'http://', 1)
         (scheme, netloc, path, query, frag) = urlparse.urlsplit(uri)
-         # urlparse doesnt understand zeo URLs and stuffs everything into path
-        (scheme, netloc, path, query, frag) = urlparse.urlsplit('http:' + path)
         if netloc:
             # TCP URL
             if ':' in netloc:
@@ -194,8 +219,7 @@ class ZConfigURIResolver(object):
     schema_xml_template = """
     <schema>
         <import package="ZODB"/>
-        <section type="ZODB.database" %s attribute="database"
-            required="yes" />
+        <multisection type="ZODB.database" attribute="databases" />
     </schema>
     """
 
@@ -204,18 +228,24 @@ class ZConfigURIResolver(object):
          # urlparse doesnt understand file URLs and stuffs everything into path
         (scheme, netloc, path, query, frag) = urlparse.urlsplit('http:' + path)
         path = os.path.normpath(path)
-        if frag:
-            name_attr = 'name="%s"' % frag
-        else:
-            name_attr = ''
-        schema_xml = self.schema_xml_template % name_attr
+        schema_xml = self.schema_xml_template
         schema = ZConfig.loadSchemaFile(StringIO(schema_xml))
         config, handler = ZConfig.loadConfig(schema, path)
-        return (path, frag), (), {}, config.database.open
+        for database in config.databases:
+            if not frag:
+                # use the first defined in the file
+                break
+            elif frag == database.name:
+                # match found
+                break
+        else:
+            raise KeyError("No database named %s found" % frag)
+        return (path, frag), (), {}, database.open
 
 
 RESOLVERS = {
     'zeo':ClientStorageURIResolver(),
     'file':FileStorageURIResolver(),
     'zconfig':ZConfigURIResolver(),
+    'memory':MappingStorageURIResolver(),
     }
