@@ -1,6 +1,17 @@
 import unittest
 
 class Base:
+
+    def failIf(self, expr, msg=None):
+        # silence stupid 2.7 stdlib deprecation
+        if expr: #pragma NO COVERAGE
+            raise self.failureException, msg
+
+    def failUnless(self, expr, msg=None):
+        # silence stupid 2.7 stdlib deprecation
+        if not expr: #pragma NO COVERAGE
+            raise self.failureException, msg
+
     def test_interpret_kwargs_noargs(self):
         resolver = self._makeOne()
         kwargs = resolver.interpret_kwargs({})
@@ -45,22 +56,6 @@ class Base:
         for name, value in args.items():
             self.assertEqual(value, 'string')
 
-    def test_bool_args(self):
-        resolver = self._makeOne()
-        f = resolver.interpret_kwargs
-        kwargs = f({'read_only':'1'})
-        self.assertEqual(kwargs, {'read_only':1})
-        kwargs = f({'read_only':'true'})
-        self.assertEqual(kwargs, {'read_only':1})
-        kwargs = f({'read_only':'on'})
-        self.assertEqual(kwargs, {'read_only':1})
-        kwargs = f({'read_only':'off'})
-        self.assertEqual(kwargs, {'read_only':0})
-        kwargs = f({'read_only':'no'})
-        self.assertEqual(kwargs, {'read_only':0})
-        kwargs = f({'read_only':'false'})
-        self.assertEqual(kwargs, {'read_only':0})
-
 class TestFileStorageURIResolver(Base, unittest.TestCase):
 
     def _getTargetClass(self):
@@ -79,6 +74,22 @@ class TestFileStorageURIResolver(Base, unittest.TestCase):
         import shutil
         shutil.rmtree(self.tmpdir)
 
+    def test_bool_args(self):
+        resolver = self._makeOne()
+        f = resolver.interpret_kwargs
+        kwargs = f({'read_only':'1'})
+        self.assertEqual(kwargs, {'read_only':1})
+        kwargs = f({'read_only':'true'})
+        self.assertEqual(kwargs, {'read_only':1})
+        kwargs = f({'read_only':'on'})
+        self.assertEqual(kwargs, {'read_only':1})
+        kwargs = f({'read_only':'off'})
+        self.assertEqual(kwargs, {'read_only':0})
+        kwargs = f({'read_only':'no'})
+        self.assertEqual(kwargs, {'read_only':0})
+        kwargs = f({'read_only':'false'})
+        self.assertEqual(kwargs, {'read_only':0})
+
     def test_call_no_qs(self):
         resolver = self._makeOne()
         k, args, kw, factory = resolver('file:///tmp/foo/bar')
@@ -253,6 +264,22 @@ class TestClientStorageURIResolver(unittest.TestCase):
         klass = self._getTargetClass()
         return klass()
 
+    def test_bool_args(self):
+        resolver = self._makeOne()
+        f = resolver.interpret_kwargs
+        kwargs = f({'read_only':'1'})
+        self.assertEqual(kwargs, {'read_only':1})
+        kwargs = f({'read_only':'true'})
+        self.assertEqual(kwargs, {'read_only':1})
+        kwargs = f({'read_only':'on'})
+        self.assertEqual(kwargs, {'read_only':1})
+        kwargs = f({'read_only':'off'})
+        self.assertEqual(kwargs, {'read_only':0})
+        kwargs = f({'read_only':'no'})
+        self.assertEqual(kwargs, {'read_only':0})
+        kwargs = f({'read_only':'false'})
+        self.assertEqual(kwargs, {'read_only':0})
+
     def test_call_tcp_no_port(self):
         resolver = self._makeOne()
         k, args, kw, factory = resolver('zeo://localhost?debug=true')
@@ -345,6 +372,11 @@ class TestZConfigURIResolver(unittest.TestCase):
 
     def test_named_database(self):
         self.tmp.write("""
+        <zodb otherdb>
+          <demostorage>
+          </demostorage>
+        </zodb>
+
         <zodb demodb>
           <mappingstorage>
           </mappingstorage>
@@ -355,7 +387,7 @@ class TestZConfigURIResolver(unittest.TestCase):
         k, args, kw, factory = resolver('zconfig://%s#demodb' % self.tmp.name)
         db = factory()
         from ZODB.MappingStorage import MappingStorage
-        self.assertTrue(isinstance(db._storage, MappingStorage))
+        self.failUnless(isinstance(db._storage, MappingStorage))
 
     def test_anonymous_database(self):
         self.tmp.write("""
@@ -363,10 +395,59 @@ class TestZConfigURIResolver(unittest.TestCase):
           <mappingstorage>
           </mappingstorage>
         </zodb>
+
+        <zodb demodb>
+          <mappingstorage>
+          </mappingstorage>
+        </zodb>
         """)
         self.tmp.flush()
         resolver = self._makeOne()
         k, args, kw, factory = resolver('zconfig://%s' % self.tmp.name)
         db = factory()
         from ZODB.MappingStorage import MappingStorage
-        self.assertTrue(isinstance(db._storage, MappingStorage))
+        self.failUnless(isinstance(db._storage, MappingStorage))
+
+    def test_database_not_found(self):
+        self.tmp.write("""
+        <zodb x>
+          <mappingstorage>
+          </mappingstorage>
+        </zodb>
+        """)
+        self.tmp.flush()
+        resolver = self._makeOne()
+        self.assertRaises(KeyError, resolver, 'zconfig://%s#y' % self.tmp.name)
+
+class TestMappingStorageURIResolver(Base, unittest.TestCase):
+
+    def _getTargetClass(self):
+        from repoze.zodbconn.resolvers import MappingStorageURIResolver
+        return MappingStorageURIResolver
+
+    def _makeOne(self):
+        klass = self._getTargetClass()
+        return klass()
+
+    def test_call_no_qs(self):
+        resolver = self._makeOne()
+        k, args, kw, factory = resolver('memory://')
+        self.assertEqual(args, ('',))
+        self.assertEqual(kw, {})
+        db = factory()
+        from ZODB.MappingStorage import MappingStorage
+        self.failUnless(isinstance(db._storage, MappingStorage))
+
+    def test_call_with_qs(self):
+        uri='memory://storagename?connection_cache_size=100&database_name=fleeb'
+        resolver = self._makeOne()
+        k, args, kw, factory = resolver(uri)
+        self.assertEqual(args, ('storagename',))
+        self.assertEqual(kw, {})
+        self.assertEqual(k,  (('storagename',),
+                              (('cache_size', 100), ('database_name', 'fleeb'),
+                               ('pool_size', 7))))
+        db = factory()
+        from ZODB.MappingStorage import MappingStorage
+        self.failUnless(isinstance(db._storage, MappingStorage))
+        
